/*
 * Cr le 8 juil. 05
 */
package com.airfrance.squalecommon.daolayer.result;

import java.util.ArrayList;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.provider.persistence.hibernate.AbstractDAOImpl;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.DAOMessages;
import com.airfrance.squalecommon.enterpriselayer.businessobject.profile.ProfileBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.SqualeReferenceBO;

/**
 * @author M400843
 *
 */
public class SqualeReferenceDAOImpl extends AbstractDAOImpl {

    /** log */
    private static Log LOG;

    /**
     * Instance singleton
     */
    private static SqualeReferenceDAOImpl instance = null;

    /** initialisation du singleton */
    static {
        instance = new SqualeReferenceDAOImpl();
    }

    /**
     * Constructeur prive
     * @throws JrafDaoException
     */
    private SqualeReferenceDAOImpl() {
        initialize(SqualeReferenceBO.class);
        if (null == LOG) {
            LOG = LogFactory.getLog(SqualeReferenceDAOImpl.class);
        }
    }

    /**
     * Retourne un singleton du DAO
     * @return singleton du DAO
     */
    public static SqualeReferenceDAOImpl getInstance() {
        return instance;
    }

    /**
     * Permet de rcuprer un nombre de SqualeReference dfinis
     * @param pSession session Hibernate
     * @param pNbLignes nombre d'audits, si <code>null</code> toutes les references seront remonts
     * @param pIndexDepart index de dpart
     * @param pIsAdmin un boolen indiquant si l'utilisateur est admin et donc si on doit rcuprer 
     * tout le rfrentiel sans prendre en compte l'attribut masqu
     * @param pUserId l'id de l'utilisateur
     * @return Collection de SqualeReference
     * @throws JrafDaoException exception DAO
     */
    public Collection findWhereScrollable(ISession pSession, Integer pNbLignes, Integer pIndexDepart, boolean pIsAdmin, Long pUserId) throws JrafDaoException {
        LOG.debug(DAOMessages.getString("dao.entry_method"));
        Collection result = null;
        StringBuffer whereClause = new StringBuffer();
        // si l'utilisateur est admin on rcupre tout le rfrentiel sans
        // prendre en compte le champ masqu, sinon on rcupre que les non-masqus
        // ou les masques mais dont l'utilisateur est admin
        if (!pIsAdmin) {
            whereClause.append("where ");
            whereClause.append(getAlias());
            whereClause.append(".hidden=0");
            whereClause.append(" or (");
            whereClause.append(getAlias());
            whereClause.append(".hidden=1 and ");
            whereClause.append(getAlias());
            whereClause.append(".applicationName in (");
            
            // select pour rcuprer les noms des applications pour lesquelles l'utilisateur a les 
            // droits de lecture
            StringBuffer selectIn = new StringBuffer("select a.name from ApplicationBO a, UserBO u where u.id=");
            selectIn.append(pUserId);
            selectIn.append(" and u.rights[a.id] is not null and u.rights[a.id].name='");
            selectIn.append(ProfileBO.MANAGER_PROFILE_NAME);

            whereClause.append(selectIn);
            whereClause.append("'))");
        }
        if ((null == pNbLignes) || (null == pIndexDepart)) {
            result = findWhere(pSession, whereClause.toString());
        } else {
            result = (Collection) super.findWhereScrollable(pSession, whereClause.toString(), pNbLignes.intValue(), pIndexDepart.intValue(), false);
        }
        LOG.debug(DAOMessages.getString("dao.exit_method"));
        return result;
    }

    /**
     * Renvoie la ligne du referentiel correspondant  application/projet (par nom)
     * @param pSession la session
     * @param pAppName le nom de l'application recherch
     * @param pProjName le nom du projet recherch
     * @return la ligne de referentiel du projet correspondant
     * @throws JrafDaoException si une erreur  lieu
     */
    public SqualeReferenceBO loadByName(ISession pSession, String pAppName, String pProjName) throws JrafDaoException {
        LOG.debug(DAOMessages.getString("dao.entry_method"));
        SqualeReferenceBO ref = null;
        String whereClause = "where ";
        whereClause += getAlias() + ".applicationName = '" + pAppName + "'";
        whereClause += " and " + getAlias() + ".projectName = '" + pProjName + "'";

        // pour un nom d'application,projet, recupere le SqualeReferenceBO
        Collection col = findWhere(pSession, whereClause);
        // qui doit etre unique !
        if (col.size() == 1) {
            ref = (SqualeReferenceBO) col.iterator().next();
        } else if (col.size() > 1) {
            throw new JrafDaoException("Too many line in referentiel");
        }

        LOG.debug(DAOMessages.getString("dao.exit_method"));
        return ref;
    }

    /**
     * @param pSession la session
     * @param pAppliName le nom de l'application recherche
     * @return l'objet rfrence correspondant au projet si il existe, null sinon
     * il ne peut pas y avoir de doublons
     * @throws JrafDaoException en cas d'chec
     */
    public SqualeReferenceBO findReferenceByAppliName(ISession pSession, String pAppliName) throws JrafDaoException {
        SqualeReferenceBO result = null;
        String whereClause = "where ";
        whereClause += getAlias() + ".applicationName = '" + pAppliName + "'";
        Collection coll = findWhere(pSession, whereClause);
        // il peut y avoir plusieurs rsultats, en fait un par projet de l'application
        // mais on ne veut rcuprer que des informations communes  tous les projets
        // type de l'audit, date de l'audit... 
        // donc on n'en renvoie qu'un seul
        if (coll != null && coll.size() >= 1) {
            result = (SqualeReferenceBO) coll.iterator().next();
        }
        return result;
    }

    /**
     * Remonte toutes les applications distinctes stockes dans le rfrentiel
     * @param pSession la session
     * @return la liste des noms
     * @throws JrafDaoException en cas d'chec
     */
    public Collection findAllDistinctAppliName(ISession pSession) throws JrafDaoException {
        Collection result = new ArrayList(0);
        // Recupration du nom de classe et de l'alias pour le Application
        int index = SqualeReferenceBO.class.getName().lastIndexOf(".");
        String className = getBusinessClass().getName().substring(index + 1);
        // On fait un distinct sur les applications pour ne pas compter plusieurs fois 
        // celles qui ont plusieurs projets
        String query = "select distinct " + getAlias() + ".applicationName " + " from " + className + " as " + getAlias();
        result = find(pSession, query);
        return result;
    }

}
