package com.airfrance.squalecommon.datatransfertobject.transform.result;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import com.airfrance.squalecommon.datatransfertobject.component.ComponentDTO;
import com.airfrance.squalecommon.datatransfertobject.result.PracticeEvolutionDTO;
import com.airfrance.squalecommon.datatransfertobject.transform.component.ComponentTransform;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AbstractComponentBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.MarkBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.PracticeResultBO;

/**
 * Transforme des donnes sous forme BO en PracticeEvolutionDTO
 */
public class PracticeEvolutionTransform {

    /**
     * @param pTabs la collection de tableau d'objets
     * @return la collection transforme
     */
    public static Collection tabCollectionToDto(Collection pTabs) {
        ArrayList results = new ArrayList();
        for (Iterator it = pTabs.iterator(); it.hasNext();) {
            Object[] currentTab = (Object[]) it.next();
            PracticeEvolutionDTO evolution = tabToDto(currentTab);
            results.add(evolution);
        }
        return results;
    }

    /**
     * @param pMarks les notes sous forme BO
     * @param pIsCurrentAudit indique si il s'agit de nouveaux composants
     * @return la collection transforme
     */
    public static Collection markCollectionToDto(Collection pMarks, boolean pIsCurrentAudit) {
        ArrayList results = new ArrayList();
        for (Iterator it = pMarks.iterator(); it.hasNext();) {
            MarkBO currentMark = (MarkBO) it.next();
            PracticeEvolutionDTO evolution = markBOToDto(currentMark, pIsCurrentAudit);
            results.add(evolution);
        }
        return results;
    }

    /**
     * @param pObject le tableau d'objets ncessaires  la construction du DTO
     * @return le tableau transform en PracticeEvolutionDTO
     */
    private static PracticeEvolutionDTO tabToDto(Object[] pObject) {
        PracticeEvolutionDTO evolution = new PracticeEvolutionDTO();
        int index = 0;
        AbstractComponentBO componentBO = (AbstractComponentBO) pObject[index++];
        ComponentDTO componentDTO = ComponentTransform.bo2Dto(componentBO);
        evolution.setComponent(componentDTO);
        PracticeResultBO practiceBO = (PracticeResultBO) pObject[index++];
        evolution.setPractice(QualityResultTransform.bo2Dto(practiceBO));
        Float mark = (Float) pObject[index++];
        evolution.setMark(mark);
        Float previousMark = (Float) pObject[index++];
        evolution.setPreviousMark(previousMark);
        return evolution;
    }

    /**
     * @param pMarkBO la note sous forme BO
     * @param pIsCurrentAudit indique si il s'agit d'un composant appartenant  l'audit
     * courant
     * @return la note transforme en PracticeEvolutionDTO
     */
    private static PracticeEvolutionDTO markBOToDto(MarkBO pMarkBO, boolean pIsCurrentAudit) {
        PracticeEvolutionDTO evolution = new PracticeEvolutionDTO();
        ComponentDTO componentDTO = ComponentTransform.bo2Dto(pMarkBO.getComponent());
        evolution.setComponent(componentDTO);
        evolution.setPractice(QualityResultTransform.bo2Dto(pMarkBO.getPractice()));
        if (pIsCurrentAudit) {
            // On change la note courante car il s'agit d'un composant de l'audit courant
            evolution.setMark(new Float(pMarkBO.getValue()));
        } else {
            // On change la note prcdente car il s'agit d'un composant de l'audit prcdent
            evolution.setPreviousMark(new Float(pMarkBO.getValue()));
        }
        return evolution;
    }

}
