//Source file: D:\\CC_VIEWS\\SQUALE_V0_0_ACT\\SQUALE\\SRC\\squaleCommon\\src\\com\\airfrance\\squalecommon\\enterpriselayer\\applicationcomponent\\ErrorApplicationComponentAccess.java

package com.airfrance.squalecommon.enterpriselayer.applicationcomponent.display;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.provider.accessdelegate.DefaultExecuteComponent;
import com.airfrance.squalecommon.datatransfertobject.component.AuditDTO;
import com.airfrance.squalecommon.datatransfertobject.component.ComponentDTO;
import com.airfrance.squalecommon.datatransfertobject.result.ErrorDTO;
import com.airfrance.squalecommon.enterpriselayer.applicationcomponent.ACMessages;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.facade.component.AuditFacade;
import com.airfrance.squalecommon.enterpriselayer.facade.quality.ErrorFacade;
import com.airfrance.squalecommon.util.messages.CommonMessages;

/**
 * <p>Title : ErrorApplicationComponentAccess.java</p>
 * <p>Description : Application component error </p>
 * <p>Copyright : Copyright (c) 2005</p>
 * <p>Company : AIRFRANCE</p>
 * Classe permettant de rcuprer les erreurs en fonction des paramtres d'audits et
 * de taches qui ont gnr les erreurs
 */
public class ErrorApplicationComponentAccess extends DefaultExecuteComponent {

    /**
     * Nombre d'audits a rcuprer par defaut
     */
    private static final Integer NB_AUDITS = new Integer(CommonMessages.getInt("audit.nombre"));

    /**
     * Index a partir duquel on souhaite rcuprer les audits par defaut
     * Les donnes sont rcupres du plus rcent au plus vieux
     */
    private static final Integer INDEX_DEPART_AUDIT =
        new Integer(CommonMessages.getInt("audit.index"));

    /** log */
    private static Log LOG = LogFactory.getLog(ErrorApplicationComponentAccess.class);

    /**
     * Permet de rcuprer les erreurs d'un projet relatifs  un audit et une tache
     * donn
     * @param pError ErrorDTO renseignant l'ID de l'audit et du projet
     * si AuditID < 0, recuperation des erreurs pour le dernier audit
     * si Task = null, recuperation des erreurs pour toutes les taches
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de depart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFBF800B5
     */
    public Collection getErrors(ErrorDTO pError, Integer pNbLignes, Integer pIndexDepart)
        throws JrafEnterpriseException {

        // Initialisation
        Collection collection = null; // retour de l'AC

        if(pError != null){

            // on verifie que les champs necessaires sont renseignes
            pError = validateErrorDTO(pError);
            if (pError!=null) { 
                // Execution de la methode
                collection = ErrorFacade.getErrors(pError, pNbLignes, pIndexDepart);
            }

        }

        return collection;
    }
    
    /**
     * Permet de rcuprer une liste d'erreurs pour un projet, un audit et 
     * plusieurs audits
     * @param pAudits liste des AuditDTO renseignant l'identifiant, sinon <code>null</code> si on souhaite 
     * les deux derniers audits de suivi
     * @param pError ErrorDTO avec un ID de composant renseigne au minimum
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de depart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFBF80105
     */
    public Collection getErrorsByAudit(
        List pAudits,
        ErrorDTO pError,
        Integer pNbLignes,
        Integer pIndexDepart)
        throws JrafEnterpriseException {

        // Initialsiation
        Collection collection = null; // retour de l'AC
        ComponentDTO project = null; // parametre pour AuditFacade
        List lastAudits = pAudits; // audits souhaites

        // Verifie si les audits sont renseignes
        if (pError.getProjectId() > 0) {
            if (pAudits == null) {

                project = new ComponentDTO();
                project.setID(pError.getProjectId());
                // recuperation des 2 derniers audits pour tous les audits
                lastAudits =
                    AuditFacade.getLastAudits(
                        project,
                        NB_AUDITS,
                        INDEX_DEPART_AUDIT,
                        null,AuditBO.FAILED);
            }

            collection = ErrorFacade.getErrorsByAudit(lastAudits, pError, pNbLignes, pIndexDepart);

        } else {
            LOG.error(ACMessages.getString("ac.exception.error.geterrorsbyaudit.negativeprojectid"));
        }

        return collection;
    }

    /**
     * Permet de rcuprer des erreurs pour un projet, un audit et une liste de taches
     * @param pTaskKeys liste des cles de taches souhaites, sinon <code>null</code> pour
     * tooutes les taches
     * @param pError ErrorDTO avec identifiant du composant au minimum
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de depart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException Exception JRAF
     * @roseuid 42CBFBF8023B
     */
    public Collection getErrorsByTask(
        List pTaskKeys,
        ErrorDTO pError,
        Integer pNbLignes,
        Integer pIndexDepart)
        throws JrafEnterpriseException {

        // Initialisation  
        Collection collection = null;

        pError = validateErrorDTO(pError);

        if (pError != null) {
            collection = ErrorFacade.getErrorsByTask(pTaskKeys, pError, pNbLignes, pIndexDepart);
        }

        return collection;
    }

    /**
     * Permet de rcuprer des erreurs pour une liste de composants et une liste 
     * d'audits donns
     * @param pAuditDTOs liste des AuditDTOs souhaits
     * @param pTaskKeys liste des cls des taches
     * @param pError ErrorDTO avec ID du composant renseign
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de depart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException Exception JRAF
     * @deprecated ne sera pas implemente
     * @roseuid 42CBFBF80336
     */
    public Collection getErrorsByAuditAndTask(
        List pAuditDTOs,
        List pTaskKeys,
        ErrorDTO pError,
        Integer pNbLignes,
        Integer pIndexDepart)
        throws JrafEnterpriseException {

        // Mise des parametres a null
        pAuditDTOs = null;
        pTaskKeys = null;
        pError = null;
        pNbLignes = null;
        pIndexDepart = null;

        Collection collection = null; // retour de l'AC
        return collection;
    }

    /**
     * Permet de completer l'objet ErrorDTO si des donnes sont manquantes
     * @param pErrorDTO ErrorDTO
     * @return ErrorDTO avec la bonne initialisation
     * @throws JrafEnterpriseException exception JRAF
     */
    private ErrorDTO validateErrorDTO(ErrorDTO pErrorDTO) throws JrafEnterpriseException {

        // initialisation
        AuditDTO lastAudit = null; // dernier audit du projet

        // teste si l'identifiant de l'audit est negatif
        if (pErrorDTO.getAuditId() < 0) {

            // initialisation du composant
            ComponentDTO project = new ComponentDTO(); // parametre de AuditFacade
            project.setID(pErrorDTO.getProjectId());

            lastAudit = AuditFacade.getLastAudit(project, null);

            if (lastAudit != null) {
                pErrorDTO.setAuditId(lastAudit.getID());
            }
        }

        // teste si l'identifiant du projet ou de l'audit est negatif
        if (pErrorDTO.getProjectId() < 0 || pErrorDTO.getAuditId() < 0) {
            LOG.error(ACMessages.getString("ac.exception.error.validateerrordto.negativeid"));
            pErrorDTO = null;
        }

        // aucun traitement si la tache est nul. Cas traite au niveau facade

        return pErrorDTO;
    }

    /**
     * @param pProjectId l'id du projet
     * @param pAuditId l'id de l'audit
     * @return les noms des tches possdant des erreurs
     * @throws JrafEnterpriseException en cas d'chec
     */
    public List getAllTasks(Long pProjectId, String pAuditId) throws JrafEnterpriseException {
        List results = new ArrayList(0);
        // Liste vide si l'audit n'est pas renseign
        if(null != pAuditId) {
            results = ErrorFacade.getAllTasks(pProjectId, new Long(Long.parseLong(pAuditId)));
        }
        return results;
    }

    /**
     * Constructeur par dfaut
     * @roseuid 42CBFBF90034
     */
    public ErrorApplicationComponentAccess() {
    }
}
