package com.airfrance.squalecommon.enterpriselayer.facade.pmd;

import java.io.InputStream;

import org.apache.commons.digester.Digester;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.RuleBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.rulechecking.pmd.PmdRuleSetBO;
import com.airfrance.squalecommon.util.xml.XmlImport;

/**
 * Parser de la configuration Pmd
 */
public class PmdConfigParser extends XmlImport {


    /** Log */
    private static Log LOG = LogFactory.getLog(PmdConfigParser.class);

    /**
     * Constructeur par dfaut
     */
    public PmdConfigParser() {
        super(LOG);
    }
    
    /**
     * Parsing du fichier de configuration Macker
     * @param pStream flux
     * @param pErrors erreurs rencontres
     * @return donnes lues
     */
    public PmdRuleSetBO parseFile(InputStream pStream, StringBuffer pErrors) {
        // Rsultat
        PmdRuleSetBO result = new PmdRuleSetBO();
        // On d'a pas de DTD  fournir
        Digester configDigester = preSetupDigester(null, null, pErrors);
        configDigester.push(new Wrapper(result));
        // On rcupre le nom du ruleSet
        configDigester.addSetProperties("ruleset");
        // On cre une rgle
        configDigester.addSetProperties("ruleset/rule");
        // On rcupre les informations supplmentaires
        configDigester.addCallMethod("ruleset/rule/properties/property", "setRuleProperty", 2, new Class[]{String.class, String.class});
        configDigester.addCallParam("ruleset/rule/properties/property", 0, "name");
        configDigester.addCallParam("ruleset/rule/properties/property", 1, "value");
        // On ajoute la rgle au ruleSet
        configDigester.addCallMethod("ruleset/rule", "addRule");
        parse(configDigester, pStream, pErrors);
        return result;
    }
    
    /**
     * 
     *
     */
    public class Wrapper {
        /** Rgle courante */
        private RuleBO mCurrentRule;

        /** Jeu de rgles */
        private PmdRuleSetBO mRuleSet;

        /**
         * Constructeur
         * @param pRuleSet ruleset
         */
        public Wrapper(PmdRuleSetBO pRuleSet) {
            mRuleSet = pRuleSet;
        }
        
        /**
         * Traitement de la balise ref
         * @param pRuleRef rfrence
         */
        public void setRef(String pRuleRef) {
            lazyCreateRule();
            int index = pRuleRef.lastIndexOf('/');
            if (index!=-1) {
                mCurrentRule.setCode(pRuleRef.substring(index+1));
            } else {
                mCurrentRule.setCode(pRuleRef);
            }
        }
        
        /**
         * Traitement des proprits
         * @param pName nom
         * @param pValue valeur
         */
        public void setRuleProperty(String pName, String pValue) {
            if (pName.equals("squaleSeverity")) {
                lazyCreateRule();
                mCurrentRule.setSeverity(pValue);
            } else if (pName.equals("squaleCategory")) {
                lazyCreateRule();
                mCurrentRule.setCategory(pValue);
            }
        }
        
        /**
         * Ajout d'une rgle
         *
         */
        public void addRule() {
            mCurrentRule.setRuleSet(mRuleSet);
            mRuleSet.addRule(mCurrentRule);
            mCurrentRule = null;
        }
        
        /**
         * Cration de la rgle courante
         *
         */
        private void lazyCreateRule() {
            if (mCurrentRule==null) {
                mCurrentRule = new RuleBO();
            }
        }
        
        /**
         * @param pLanguage langage
         */
        public void setLanguage(String pLanguage) {
            mRuleSet.setLanguage(pLanguage);
        }

        /**
         * @param pName nom
         */
        public void setName(String pName) {
            mRuleSet.setName(pName);
        }

    }
}
