//Source file: D:\\CC_VIEWS\\SQUALE_V0_0_ACT\\SQUALE\\SRC\\squaleCommon\\src\\com\\airfrance\\squalecommon\\enterpriselayer\\facade\\ErrorFacade.java

package com.airfrance.squalecommon.enterpriselayer.facade.quality;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.commons.exception.JrafEnterpriseException;
import com.airfrance.jraf.helper.PersistenceHelper;
import com.airfrance.jraf.provider.persistence.hibernate.facade.FacadeHelper;
import com.airfrance.jraf.spi.enterpriselayer.IFacade;
import com.airfrance.jraf.spi.persistence.IPersistenceProvider;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.result.ErrorDAOImpl;
import com.airfrance.squalecommon.datatransfertobject.component.AuditDTO;
import com.airfrance.squalecommon.datatransfertobject.result.ErrorDTO;
import com.airfrance.squalecommon.datatransfertobject.transform.result.ErrorTransform;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.ErrorBO;

/**
 */
public class ErrorFacade implements IFacade {

    /**
     * provider de persistence
     */
    private static final IPersistenceProvider PERSISTENTPROVIDER = PersistenceHelper.getPersistenceProvider();

    /**
     * Permet de rcuprer les erreurs d'un projet relatifs  un audit et une 
     * tache
     * et un audit donn
     * @param pError ErrorDTO avec ID du projet, de l'audit et la cl de la tache renseign
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de dpart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB30180
     */
    public static Collection getErrors(ErrorDTO pError, Integer pNbLignes, Integer pIndexDepart) throws JrafEnterpriseException {

        // Initialisation des parametres
        Collection collection = null; // retour de la facade
        List tasks = new ArrayList(); // parametre de la methode appelee

        // Initialisation des parametres de la methodes

        if (pError.getTaskName() != null) {
            tasks.add(pError.getTaskName());
        } else {
            tasks = null;
        }

        collection = getErrorsByTask(tasks, pError, pNbLignes, pIndexDepart);

        return collection;

    }

    /**
     * Permet de rcuprer une liste d'erreurs pour un projet, un audit et 
     * plusieurs audits
     * @param pAuditDTOs liste d'AuditDTOs
     * @param pError ErrorDTO avec ID du projet et la cl de la tache renseign
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de dpart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB3019F
     */
    public static Collection getErrorsByAudit(List pAuditDTOs, ErrorDTO pError, Integer pNbLignes, Integer pIndexDepart) throws JrafEnterpriseException {

        // Initialisation
        Collection collection = new ArrayList(); // retour de la facade
        Collection errorsPerAudit = null; // retour de getErrors

        // Utilisation n fois de la methode getErrors()
        Iterator auditIterator = pAuditDTOs.iterator();
        AuditDTO auditTemp = null;
        while (auditIterator.hasNext()) {

            // initialisation du retour de la facade
            if (collection == null) {
                collection = new ArrayList();
            }

            auditTemp = (AuditDTO) auditIterator.next();
            pError.setAuditId(auditTemp.getID());

            errorsPerAudit = getErrors(pError, pNbLignes, pIndexDepart);
            if (errorsPerAudit == null) {
                collection = null;
            } else {
                collection.addAll(errorsPerAudit);
            }

        }

        return collection;
    }

    /**
     * Permet de rcuprer des erreurs pour un projet, un audit et une liste de 
     * taches
     * @param pTaskKeys liste des cls des taches souhaitees sinon <code>null</code> 
     * pour toutes les taches, sinon <code>null</code> pour toutes les taches
     * @param pError ErrorDTO avec ID du projet et de l'audit renseign
     * @param pNbLignes nombre de lignes
     * @param pIndexDepart index de dpart
     * @return Collection de ErrorDTO
     * @throws JrafEnterpriseException exception JRAF
     * @roseuid 42CBFFB301AB
     */
    public static Collection getErrorsByTask(List pTaskKeys, ErrorDTO pError, Integer pNbLignes, Integer pIndexDepart) throws JrafEnterpriseException {

        // Initialisation 
        Collection errorDTOs = null; // retour de la facade
        Collection errorBOs = null; // retour de la DAO
        Long auditID = new Long(pError.getAuditId()); // identifiant de l'audit
        Long projectId = new Long(pError.getProjectId()); // identifiant du composant

        ISession session = null;

        try {
            // rcupration d'une se session
            session = PERSISTENTPROVIDER.getSession();

            ErrorDAOImpl errorDAO = ErrorDAOImpl.getInstance();

            // rcupration des ErrorBO correspondant  l'audit, au projet et  la tache
            errorBOs = (Collection) errorDAO.findWhere(session, auditID, projectId, pTaskKeys, pNbLignes, pIndexDepart);

            Iterator errorIterator = errorBOs.iterator();
            ErrorBO errorTemp = null;
            // initialisation de la collection de retour
            errorDTOs = new ArrayList();
            while (errorIterator.hasNext()) {
                // pour chaque ErrorBO
                errorTemp = (ErrorBO) errorIterator.next();
                // transformation et ajout  la collection de retour
                errorDTOs.add(ErrorTransform.bo2Dto(errorTemp));
            }

        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ErrorFacade.class.getName() + ".getErrorsByTask");
        } finally {
            FacadeHelper.closeSession(session, ErrorFacade.class.getName() + ".getErrorsByTask");
        }

        return errorDTOs;
    }

    /**
     * Constructeur vide
     * @roseuid 42CBFFB301C9
     */
    private ErrorFacade() {
    }

    /**
     * @param pAuditId l'id de l'audit
     * @param pProjectId l'id du projet
     * @return un boolen indiquant si il y eu des erreurs
     * @throws JrafEnterpriseException en cas d'chec
     */
    public static int getNumberOfErrors(Long pAuditId,Long pProjectId) throws JrafEnterpriseException {
        int result = 0;
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            ErrorDAOImpl errorDAO = ErrorDAOImpl.getInstance();
            // rcupration du nombre d'erreurs correspondant  l'audit
            result = errorDAO.getNumberOfErrorsWhere(session,pAuditId,pProjectId, null).intValue();
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ErrorFacade.class.getName() + ".getNumberOfErrors");
        } finally {
            FacadeHelper.closeSession(session, ErrorFacade.class.getName() + ".getNumberOfErrors");
        }
        return result;
    }

    /**
     * @param pAuditId l'id de l'audit
     * @param pProjectId l'id du projet
     * @return le nombre d'erreurs provoqu par le projet par niveau :
     * 0 -> ErrorBO.CRITICITY_FATAL
     * 1 -> ErrorBO.CRITICITY_WARNING
     * 1 -> ErrorBO.CRITICITY_LOW
     * @throws JrafEnterpriseException en cas d'chec
     */
    public static Integer[] getErrorsRepartition(Long pAuditId, Long pProjectId) throws JrafEnterpriseException {
        final int NB_ERRORS_LEVELS = 3;
        Integer[] result = new Integer[NB_ERRORS_LEVELS];
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            ErrorDAOImpl errorDAO = ErrorDAOImpl.getInstance();
            // rcupration du nombre d'erreurs correspondant  l'audit par niveau
            result[0] = errorDAO.getNumberOfErrorsWhere(session,pAuditId,pProjectId, ErrorBO.CRITICITY_FATAL);
            result[1] = errorDAO.getNumberOfErrorsWhere(session,pAuditId,pProjectId, ErrorBO.CRITICITY_WARNING);
            result[2] = errorDAO.getNumberOfErrorsWhere(session,pAuditId,pProjectId, ErrorBO.CRITICITY_LOW);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ErrorFacade.class.getName() + ".getNumberOfErrors");
            result = null;
        } finally {
            FacadeHelper.closeSession(session, ErrorFacade.class.getName() + ".getNumberOfErrors");
        }
        return result;
    }

    /**
     * @param pAuditId l'id de l'audit
     * @param pProjectId l'id du projet
     * @return les noms des tches en chec
     * @throws JrafEnterpriseException en cas d'chec
     */
    public static List getFailedTasks(Long pAuditId,Long pProjectId) throws JrafEnterpriseException {
        List results = new ArrayList(0);
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            ErrorDAOImpl errorDAO = ErrorDAOImpl.getInstance();
            // rcupration des noms des tches
            results = errorDAO.getTasksNameWhere(session,pAuditId,pProjectId, ErrorBO.CRITICITY_FATAL);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ErrorFacade.class.getName() + ".getNumberOfErrors");
        } finally {
            FacadeHelper.closeSession(session, ErrorFacade.class.getName() + ".getNumberOfErrors");
        }
        return results;
    }

    /**
     * @param pProjectId l'id du projet
     * @param pAuditId l'id de l'audit
     * @return les noms des tches possdant des erreurs
     * @throws JrafEnterpriseException en cas d'chec
     */
    public static List getAllTasks(Long pProjectId, Long pAuditId) throws JrafEnterpriseException {
        List results = new ArrayList(0);
        ISession session = null;
        try {
            // rcupration d'une session
            session = PERSISTENTPROVIDER.getSession();
            ErrorDAOImpl errorDAO = ErrorDAOImpl.getInstance();
            // rcupration des noms des tches
            results = errorDAO.getTasksNameWhere(session,pAuditId,pProjectId, null);
        } catch (JrafDaoException e) {
            FacadeHelper.convertException(e, ErrorFacade.class.getName() + ".getAllTasks");
        } finally {
            FacadeHelper.closeSession(session, ErrorFacade.class.getName() + ".getAllTasks");
        }
        return results;
    }
}
