package com.airfrance.squalecommon.enterpriselayer.facade.rule;

import org.python.core.PyException;
import org.python.core.PyFloat;
import org.python.core.PyFunction;
import org.python.core.PyObject;
import org.python.util.PythonInterpreter;

/**
 * Interprteur pour les fonctions de pondration de la grille qualit
 */
public class WeightFunctionInterpreter {

    /** Interpreteur Jython */
    private PythonInterpreter mInterpreter;

    /**
     * Constructeur par dfaut
     */
    public WeightFunctionInterpreter() {
        mInterpreter = new PythonInterpreter();
        mInterpreter.exec("from math import *\n");
    }

    /**
     * Constructeur
     * @param pInterpreter l'interpreteur Jython
     */
    public WeightFunctionInterpreter(PythonInterpreter pInterpreter) {
        if (mInterpreter == null) {
            mInterpreter = new PythonInterpreter();
            mInterpreter.exec("from math import *\n");
        } else {
            mInterpreter = pInterpreter;
        }
    }

    /**
     * @param pFunction la fonction  vrifier
     * @throws WeightFunctionException si erreur de syntaxe
     */
    public void checkSyntax(String pFunction) throws WeightFunctionException {
        try {
            PyObject pyFunction = mInterpreter.eval(pFunction);
            // Le rsultat de l'valuation doit tre un objet de type pyFunction
            if (!(pyFunction instanceof PyFunction)) {
                // On lance une exception
                String error = RuleMessages.getString("function.eval.badtype", new Object[] { pyFunction.getClass().getName()});
                throw new WeightFunctionException(error);
            }
            // La fonction doit tre  une variable
            // TODO??
        } catch (PyException pe) {
            throw new WeightFunctionException(pe);
        }
    }

    /**
     * @param pFunction la fonction  appliquer
     * @param pMark la note
     * @return pFunction(pMark)
     * @throws WeightFunctionException si erreur Jython
     */
    public float exec(String pFunction, float pMark) throws WeightFunctionException {
        PyFunction pythonResult = (PyFunction) mInterpreter.eval(pFunction);
        float fMark = -1;
        try {
            PyFloat res = (PyFloat) pythonResult.__call__(new PyFloat(pMark));
            fMark = (float) res.getValue();
        } catch (Throwable t) {
            // Renvoi d'une exception si la couche Jython rencontre une erreur
            throw new WeightFunctionException(t);
        }
        return fMark;
    }

    /**
     * @return l'interprteur Jython
     */
    public PythonInterpreter getInterpreter() {
        return mInterpreter;
    }

}
