package com.airfrance.squaleweb.applicationlayer.action.component.parameters;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessages;

import com.airfrance.squaleweb.applicationlayer.action.accessRights.ReaderAction;
import com.airfrance.squaleweb.applicationlayer.action.component.CreateProjectAction;
import com.airfrance.squaleweb.applicationlayer.formbean.component.parameters.AbstractParameterForm;
import com.airfrance.squaleweb.applicationlayer.formbean.creation.CreateProjectForm;
import com.airfrance.welcom.struts.bean.WActionForm;
import com.airfrance.welcom.struts.transformer.WTransformerFactory;

/**
 * Interface pour les actions associes  la configuration d'une tche
 */
public class CreateParametersAction extends ReaderAction {

    /**
    * Permet de remplir le bean de la tche et de l'enregistrer en session si besoin.
    * 
    * @param pMapping le mapping.
    * @param pForm le formulaire  lire.
    * @param pRequest la requte HTTP.
    * @param pResponse la rponse de la servlet.
    * @return l'action  raliser.
    */
    public ActionForward fill(ActionMapping pMapping, ActionForm pForm, HttpServletRequest pRequest, HttpServletResponse pResponse) {
        ActionForward forward = null;
        ActionMessages errors = new ActionMessages();
        try {
            forward = pMapping.findForward("config");
            // Affectation au projet courant
            CreateProjectForm project = (CreateProjectForm) pRequest.getSession().getAttribute("createProjectForm");
            pForm.reset(pMapping, pRequest);
            WTransformerFactory.objToForm(((AbstractParameterForm) pForm).getTransformer(), ((WActionForm) pForm), getTransformerParameters(project, pRequest));
            // On ajout le nom du formulaire pour pouvoir le rcuprer
            project.getTaskForms().add(((AbstractParameterForm) pForm).getNameInSession());
        } catch (Exception e) {
            // Traitement factoris des exceptions
            handleException(e, errors, pRequest);
            // Routage vers la page d'erreur
            forward = pMapping.findForward("total_failure");
        }
        if (!errors.isEmpty()) {
            saveMessages(pRequest, errors);
        }
        //On est pass par un menu donc on rinitialise le traceur
        resetTracker(pRequest);
        return null;
    }

    /**
     * Ajoute les paramtres au projet
     * @param pMapping le mapping
     * @param pForm le formulaire
     * @param pRequest la requte
     * @param pResponse la rponse
     * @return l'action  raliser
     */
    public ActionForward addParameters(ActionMapping pMapping, ActionForm pForm, HttpServletRequest pRequest, HttpServletResponse pResponse) {
        ActionForward forward = null;
        ActionMessages errors = new ActionMessages();
        // Affectation au projet courant
        CreateProjectForm project = (CreateProjectForm) pRequest.getSession().getAttribute("createProjectForm");
        try {
            forward = pMapping.findForward("configure");

            // On modifie les paramtres du projet pour cette tche
            WTransformerFactory.formToObj(((AbstractParameterForm) pForm).getTransformer(), (WActionForm) pForm, getTransformerParameters(project, pRequest));

            // On sauvegarde le projet
            new CreateProjectAction().saveProject(pMapping, project, pRequest, pResponse);
        } catch (Exception e) {
            // Traitement factoris des exceptions
            handleException(e, errors, pRequest);
        }
        if (!errors.isEmpty()) {
            saveMessages(pRequest, errors);
        }
        //On est pass par un menu donc on rinitialise le traceur
        resetTracker(pRequest);
        return forward;
    }

    /**
     * @param pProject le formulaire reprsentant le projet
     * @param pRequest la requte
     * @return les paramtres ncessaires au transformer
     * @throws Exception si erreur
     */
    public Object[] getTransformerParameters(CreateProjectForm pProject, HttpServletRequest pRequest) throws Exception {
        return new Object[] { pProject.getParameters()};
    }

    /**
     * Permet de dconfigurer la tche.
     * 
     * @param pMapping le mapping.
     * @param pForm le formulaire  lire.
     * @param pRequest la requte HTTP.
     * @param pResponse la rponse de la servlet.
     * @return l'action  raliser.
     */
    public ActionForward removeParameters(ActionMapping pMapping, ActionForm pForm, HttpServletRequest pRequest, HttpServletResponse pResponse) {
        ActionForward forward = null;
        ActionMessages errors = new ActionMessages();
        try {
            forward = pMapping.findForward("configure");
            // Affectation au projet courant
            CreateProjectForm project = (CreateProjectForm) pRequest.getSession().getAttribute("createProjectForm");
            // On supprime toutes les entres lies  la tche dans les paramtres du projet
            String[] constants = ((AbstractParameterForm) pForm).getParametersConstants();
            for(int i=0; i<constants.length; i++) {
                project.getParameters().getParameters().remove(constants[i]);
            }
            // On sauvegarde le projet
            new CreateProjectAction().saveProject(pMapping, project, pRequest, pResponse);
        } catch (Exception e) {
            // Traitement factoris des exceptions
            handleException(e, errors, pRequest);
            // Routage vers la page d'erreur
            forward = pMapping.findForward("total_failure");
        }
        if (!errors.isEmpty()) {
            saveMessages(pRequest, errors);
        }
        //On est pass par un menu donc on rinitialise le traceur
        resetTracker(pRequest);
        return forward;
    }
}
