package com.airfrance.squaleweb.applicationlayer.formbean.creation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionMapping;

import com.airfrance.squalecommon.datatransfertobject.component.parameters.MapParameterDTO;
import com.airfrance.squalecommon.datatransfertobject.config.TaskDTO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squaleweb.applicationlayer.formbean.ActionIdFormSelectable;

/**
 * Form bean for a Struts application.
 * @version 1.0
 * @author
 */
public class CreateProjectForm extends ActionIdFormSelectable {

    /**
     * Les paramtres du projet
     */
    private MapParameterDTO mParameters;

    /** Grille qualit */
    private String mQualityGrid;

    /**
     * Profil de l'application (Java, CPP, ...)
     */
    private String mProfile;

    /**
     * Type de rcupration des sources de l'application
     */
    private String mSourceManagement;

    /**
     * Liste des tches configurables et standards
     */
    private List mStandardTasks;

    /**
     * Liste des tches configurables et dont la configuration
     * est avance
     */
    private List mAdvancedTasks;

    /**
     * Liste des noms des beans associs aux paramtres
     * du projet.
     */
    private Set mTaskForms;

    /**
     * Le statut du projet
     */
    private int mStatus;

    /**
     * Constructeur
     *
     */
    public CreateProjectForm() {
        setProjectId("-1");
        mParameters = new MapParameterDTO();
        mQualityGrid = "";
        mProfile = "";
        mSourceManagement = "";
        mStandardTasks = new ArrayList();
        mAdvancedTasks = new ArrayList();
        mTaskForms = new TreeSet();
        mStatus = ProjectBO.ACTIVATED;
    }

    /**
     * Reset du formulaire
     */
    public void reset() {
        setProjectId("-1");
        setProjectName("");
        mParameters = new MapParameterDTO();
        mQualityGrid = "";
        mProfile = "";
        mSourceManagement = "";
        mStandardTasks = new ArrayList();
        mAdvancedTasks = new ArrayList();
        mTaskForms = new TreeSet();
        mStatus = ProjectBO.ACTIVATED;
    }

    /**
     * Access method for the mProfile property.
     * 
     * @return   the current value of the mProfile property
     */
    public String getProfile() {
        return mProfile;
    }

    /**
     * Sets the value of the mProfile property.
     * 
     * @param pProfile the new value of the mProfile property
     */
    public void setProfile(String pProfile) {
        mProfile = pProfile;
    }

    /**
     * @return grille qualit
     */
    public String getQualityGrid() {
        return mQualityGrid;
    }

    /**
     * @param pString grille qualit
     */
    public void setQualityGrid(String pString) {
        mQualityGrid = pString;
    }

    /**
     * @return le rcuprateur de sources
     */
    public String getSourceManagement() {
        return mSourceManagement;
    }

    /**
     * @param pSourceManagement le rcuprateur de sources
     */
    public void setSourceManagement(String pSourceManagement) {
        mSourceManagement = pSourceManagement;
    }

    /**
     * @return la liste des tches de configuration standard
     */
    public List getStandardTasks() {
        return mStandardTasks;
    }

    /**
     * @param pTasks la liste des tches de configuration standard
     */
    public void setStandardTasks(List pTasks) {
        mStandardTasks = pTasks;
    }

    /**
     * @return la liste des tches de configuration avance
     */
    public List getAdvancedTasks() {
        return mAdvancedTasks;
    }

    /**
     * @param pTasks la liste des tches de configuration avance
     */
    public void setAdvancedTasks(List pTasks) {
        mAdvancedTasks = pTasks;
    }

    /**
     * Dispatche les tches dans les bonnes listes
     * @param pTasks les tches configurables
     */
    public void setTasks(Collection pTasks) {
        // On reset les listes
        mStandardTasks.clear();
        mAdvancedTasks.clear();
        TaskDTO task = null;
        for (Iterator it = pTasks.iterator(); it.hasNext();) {
            task = (TaskDTO) it.next();
            if (task.isStandard()) {
                mStandardTasks.add(task);
            } else {
                mAdvancedTasks.add(task);
            }
        }
    }

    /**
     * @return les paramtres
     */
    public MapParameterDTO getParameters() {
        return mParameters;
    }

    /**
     * @param pParameters les nouveaux paramtres
     */
    public void setParameters(MapParameterDTO pParameters) {
        mParameters = pParameters;
    }

    /**
     * @return les nom des beans des tches configures
     */
    public Set getTaskForms() {
        return mTaskForms;
    }

    /**
     * @param pTaskForms la liste des nom des beans des tches configures
     */
    public void setTaskForms(Set pTaskForms) {
        mTaskForms = pTaskForms;
    }

    /**
     * Obtention d'une tche par son nom
     * @param pTaskName nom de la tche
     * @return tche correspondante
     */
    public TaskDTO getConfigurableTask(String pTaskName) {
        ArrayList configurableTasks = new ArrayList(mStandardTasks);
        configurableTasks.addAll(mAdvancedTasks);
        TaskDTO result = null;
        for (Iterator it = configurableTasks.iterator(); it.hasNext() && (result == null);) {
            TaskDTO task = (TaskDTO) it.next();
            if (task.getName().equals(pTaskName)) {
                result = task;
            }
        }
        return result;
    }

    /**
     * @return le statut
     */
    public int getStatus() {
        return mStatus;
    }

    /**
     * @param pStatus le statut
     */
    public void setStatus(int pStatus) {
        mStatus = pStatus;
    }

    /**
     * Mthode de validation pour vrifier que le nom mis  l'application ne contient pas de caractrs spciaux.
     * La liste des caractres spciaux est dfini par le pattern situ dans la classe mre
     * @param pMapping le mapping hibernate
     * @param pRequest la requete
     */
    public void wValidate(ActionMapping pMapping, HttpServletRequest pRequest) {
        // Le nom du projet
        if(getProjectName().length() == 0) {
            addError("projectName", new ActionError("error.field.required"));
        }else if (!isAValidName(getProjectName())) {
            addError("projectName", new ActionError("error.name.containsInvalidCharacter"));
        }
        // Le rcuprateur de source
        if(getSourceManagement().length() == 0) {
            addError("sourceManagement", new ActionError("error.field.required")); 
        }
        // Le profil
        if(getProfile().length() == 0) {
            addError("profile", new ActionError("error.field.required")); 
        }
        // La grille qualit
        if(getQualityGrid().length() == 0) {
            addError("qualityGrid", new ActionError("error.field.required")); 
        }
    }

}
