//Source file: D:\\cc_views\\squale_v0_0_act\\squale\\src\\squalix\\src\\com\\airfrance\\squalix\\tools\\clearcase\\ClearCaseCleanerTask.java

package com.airfrance.squalix.tools.clearcase.task;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalix.core.AbstractTask;
import com.airfrance.squalix.core.TaskException;
import com.airfrance.squalix.tools.clearcase.configuration.ClearCaseConfiguration;
import com.airfrance.squalix.tools.clearcase.configuration.ClearCaseMessages;
import com.airfrance.squalix.util.process.ProcessErrorHandler;
import com.airfrance.squalix.util.process.ProcessManager;

/**
 * Nettoie l'environnement de travail aprs avoir l'analyse complte d'un 
 * projet.<br>
 * Dmonte la vue ClearCase notamment.<br>
 * Elle est la dernire tche excute sur un projet avant le calcul des 
 * rsultats qualit.
 * @author m400832
 * @version 2.1
 */
public class ClearCaseCleanerTask extends AbstractTask implements ProcessErrorHandler {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(ClearCaseCleanerTask.class);

    /**
     * Instance de configuration.
     */
    private ClearCaseConfiguration mConfiguration;

    /**
     * Constructeur.
     * @roseuid 42B97E86009E
     */
    public ClearCaseCleanerTask() {
        mName = "ClearCaseCleanerTask";
    }
    
    // Cette tache n'a pas d'influence dans le calcul de la taille max du file system

    /**
     * Mthode de lancement de la tche de dmontage.
     * @throws TaskException en cas d'chec
     *  @roseuid 42AE95AB03C8
     */
    public void execute() throws TaskException {
        /* rcupre la configuration clearcase */
        try {
            mConfiguration = new ClearCaseConfiguration(mProject, mAudit);
            /* dmontage */
            umount();
        } catch (Exception e) {
            throw new TaskException(e);
        }
    }

    /**
     * Mthode cre pour viter une pieuvre.
     * @see #cleanView()
     * @throws Exception exception.
     */
    private void umount() throws Exception {
        /* suppression de la vue */
        if (cleanView()) {
            LOGGER.info(ClearCaseMessages.getString("logs.view.umounted"));

            /* si la vue n'a pu tre dmonte */
        } else {
            /* exception lance */
            throw new Exception(ClearCaseMessages.getString("exception.view.umount_failed"));
        }
    }

    /**
     * Supprime / dmonte la vue ClearCase
     * @throws Exception si la commande UNIX produit des erreurs.
     * @return <code>true</code> si la vue est correctement dmonte, <code>
     * false</code> sinon.
     */
    protected boolean cleanView() throws Exception {
        /* instanciation du process et excution */
        ProcessManager mgr = new ProcessManager(mConfiguration.getUmountViewCommand(), null, null);

        /* si le processus se termine correctement */
        return (mgr.startProcess(this) == 0);
    }
    /** (non-Javadoc)
     * @see com.airfrance.squalix.util.process.ProcessErrorHandler#processError(java.lang.String)
     */
    public void processError(String pErrorMessage) {
        initError(pErrorMessage);
    }
}
