package com.airfrance.squalix.tools.cpptest;

import java.io.File;
import java.io.IOException;
import java.util.Collection;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalix.util.file.FileUtility;


/**
 * Espace de travail utilis pour la gnration CppTest
 * CppTest utilise des rpertoires pendant son excution, un rpertoire
 * servant  gnrer les donnes du projet CppTest, l'autre servant
 *  gnrer les rapports.
 * Ces rpertoires sont transitoires.
 */
public class CppTestWorkSpace {
    /** Extension d'un rapport CppTest */
    static final String REPORT_EXTENSION = ".xml";
    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(CppTestWorkSpace.class);
    /** Rpertoire racine */
    private File mRootDir;
    /** Rpertoire des rapports */
    private File mReportDirectory;
    /** Project CppTest */
    private File mProjectFile;
    
    /**
     * Constructeur
     * @param pRootDir rpertoire racine
     * @throws IOException si erreur de cration des rpertoires
     */
    public CppTestWorkSpace(File pRootDir) throws IOException {
        // Cration du rpertoire correspondant
        mRootDir = pRootDir;
        createDirectory(pRootDir);
        // Cration du rpertoire pour le projet
        File projectDir = new File(mRootDir, "project");
        createDirectory(projectDir);
        mProjectFile = new File(projectDir, "project.cpf");
        // Cration du rpertoire pour le rapport
        mReportDirectory = new File(mRootDir, "report");
        createDirectory(mReportDirectory);
    }
    
    /**
     * Cration du rpertoire
     * @param pDirectory rpertoire  crer
     * @throws IOException si la cration ne peut se faire
     */
    private void createDirectory(File pDirectory) throws IOException {
        if (false==pDirectory.mkdirs()) {
            String message = CppTestMessages.getString("error.create_directory", pDirectory.getAbsolutePath());
            LOGGER.error(message);
            throw new IOException(message);
        }
    }
    
    /**
     * Nettoyage des rprtoires
     * Le rpertoire racine est dtruit rcursivement
     */
    public void cleanup() {
        // Effacement du rpertoire de rapport
        FileUtility.deleteRecursively(mRootDir);
    }

    /**
     * Recherche des rapports
     * @return rapports au format XML gnrs par CppTest
     */
    public Collection getReportFiles() {    
        // Recherche des rapports
        Collection files = FileUtility.findFilesWithSuffix(getReportDirectory(), REPORT_EXTENSION);
        return files;
    }

    /**
     * @return projet CppTest
     */
    public File getProjectFile() {
        return mProjectFile;
    }

    /**
     * @return rpertoire de gnration de rapport
     */
    public File getReportDirectory() {
        return mReportDirectory;
    }

}