package com.airfrance.squalix.tools.macker;

import java.io.File;
import java.io.IOException;
import java.util.List;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AbstractComponentBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ClassBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.JspBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squalix.util.file.FileUtility;
import com.airfrance.squalix.util.file.JspFileUtility;
import com.airfrance.squalix.util.parser.J2EEParser;

/**
 *
 */
public class J2eeStorageListener extends JavaStorageListener {

    /** La liste des chemins absolus vers les pages jsps */
    private List mJspPaths;

    /** Rpertoire contenant les jsps compiles */
    private String mCompiledJsp;

    /**
     * Constructeur par dfaut
     * 
     * @param pSession la session
     * @param pProject le projet  auditer
     * @param pConfiguration la configuration Macker
     */
    public J2eeStorageListener(ISession pSession, ProjectBO pProject, MackerConfiguration pConfiguration) {
        super(pSession, pProject, pConfiguration);
        this.mParser = new J2EEParser(pProject);
        mJspPaths = pConfiguration.getJsps();
        mCompiledJsp = pConfiguration.getJspRoot();
    }

    /**
     * Crer et fait persister la page JSP dont le nom absolu
     * est <code>pFullName</code> si celle-ci appartient au
     * projet  auditer et n'appartient pas  un des rpertoires exclus.
     * 
     * @param pFullName le nom absolu de la classe
     * @throws IOException si erreur de flux
     * @throws JrafDaoException si erreur de persistance
     * @return le composant persist 
     */
    protected AbstractComponentBO getComponent(String pFullName) throws IOException, JrafDaoException {
        AbstractComponentBO jspOrClassBO = null;
        String absoluteFileName = null;
        // On rcupre le nom absolu du fichier compil associ  la classe
        String classFileName = FileUtility.getClassFileName(mFilesToAnalyse, pFullName);
        if (null != classFileName) { // On a trouv le .class associ
            File compiledJspDir = new File(mCompiledJsp);
            if (classFileName.startsWith(compiledJspDir.getCanonicalPath())) { // Il s'agit d'une jsp compile
                // On rcupre le nom absolu du fichier parmis les jsps
                // si elle existe et peut tre persiste
                absoluteFileName = getAbsoluteFileName(pFullName);
                if (null != absoluteFileName) { // Il faut crer une jsp
                    // On rcupre la jsp
                    String name = absoluteFileName.substring(absoluteFileName.lastIndexOf("/") + 1, absoluteFileName.lastIndexOf("."));
                    int id = JspFileUtility.getJspDirectoryId(mJspPaths, pFullName.substring(0, pFullName.indexOf(".")));
                    File rootDir = new File((String)mJspPaths.get(id));
                    String relativeRootDir = FileUtility.getRelativeFileName(rootDir.getAbsolutePath(), mViewPath);
                    String relativeFileName = FileUtility.getRelativeFileName(absoluteFileName, mViewPath);
                    jspOrClassBO = ((J2EEParser) mParser).getJsp(name, relativeFileName, relativeRootDir, id);
                    // On fait persister la jsp
                    jspOrClassBO = (JspBO) mRepository.persisteComponent(jspOrClassBO);
                }
            } else {
                // On vrifie que le nom absolu du fichier .java correspondant
                // appartient  la liste des fichiers qui peuvent tre analyss
                String relativeFileName = isInclude(pFullName);
                if (null != relativeFileName) { // Il faut crer une classe
                    //On rcupre la classe
                    jspOrClassBO = mParser.getClass(pFullName, relativeFileName);
                    // On fait persister la classe
                    jspOrClassBO = (ClassBO) mRepository.persisteComponent(jspOrClassBO);
                }
            }
        }
        return jspOrClassBO;
    }

    /**
     * True si le nom absolu du fichier jsp correspondant  la classe <code>pFullName</code>
     * appartient  la liste des fichiers qui peuvent tre analyss
     * @param pFullName le nom absolu de la classe JSP
     * @return le nom absolu du fichier dans lequel la jsp est dfinie
     * @throws IOException si erreur de flux
     */
    protected String getAbsoluteFileName(String pFullName) throws IOException {
        String absoluteFileName = null;
        // On rcupre le nom absolu du fichier compil associ  la classe
        absoluteFileName = JspFileUtility.getAbsoluteJspFileName(mJspPaths, pFullName);
        if (null != absoluteFileName) {
            // On vrifie que la jsp peut tre persiste
            // On remplace le sparateur par celui par dfaut
            absoluteFileName = absoluteFileName.replaceAll("\\\\", "/");
            String parentName = absoluteFileName.substring(0, absoluteFileName.lastIndexOf("/"));
            if (!mIncludedFiles.contains(absoluteFileName)) {
                absoluteFileName = null;
            }
        }
        return absoluteFileName;
    }
}