//Source file: D:\\cc_views\\squale_v0_0_act\\squale\\src\\squalix\\src\\com\\airfrance\\squalix\\tools\\mccabe\\McCabePersistor.java

package com.airfrance.squalix.tools.mccabe;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.jraf.commons.exception.JrafDaoException;
import com.airfrance.jraf.spi.persistence.ISession;
import com.airfrance.squalecommon.daolayer.result.MeasureDAOImpl;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.AuditBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ListParameterBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.parameters.ParametersConstants;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAClassMetricsBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAJspMetricsBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAMethodMetricsBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.result.mccabe.McCabeQAProjectMetricsBO;
import com.airfrance.squalix.core.TaskData;
import com.airfrance.squalix.util.buildpath.BuildProjectPath;
import com.airfrance.squalix.util.csv.CSVParser;
import com.airfrance.squalix.util.file.FileUtility;
import com.airfrance.squalix.util.file.ToolsReportsUtility;
import com.airfrance.squalix.util.parser.LanguageParser;
import com.airfrance.squalix.util.repository.ComponentRepository;

/**
 * Objet charg de faire persister les composants identifis par McCabe (mthodes 
 * et classes) ainsi que les rsultats
 * @author m400842
 * @version 1.0
 */
public class McCabePersistor {

    /** L'extension d'une page jsp */
    public static final String JSP_EXTENSION = ".jsp";

    /** Signature McCabe de la mthode d'une JSP */
    public static final String JSP_METHOD_REGEXP = "._jspService\\(HttpServletRequest,HttpServletResponse\\)";

    /**
     * Metriques des JSP
     */
    private Map mJspResults = new HashMap();

    /**
     * Nombre de mthodes
     */
    private int mNumberOfMethods = 0;
    /**
     * Nombre de classes
     */
    private int mNumberOfClasses = 0;

    /** le nom de la tache rel, pour diffrencier java et cpp */
    private String mTaskName;

    /**
     * Configuration
     */
    private McCabeConfiguration mConfiguration;

    /**
     * Adaptateur
     */
    private McCabeAdaptator mAdaptator;

    /**
     * Chemin du fichier  parser
     */
    private String mReportFileName = null;

    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(McCabePersistor.class);

    /**
     * Audit durant lequel l'analyse est effectue
     */
    private AuditBO mAudit = null;

    /**
     * Projet sur lequel est ralise l'analyse.
     */
    private ProjectBO mProject = null;

    /**
     * Session Persistance
     */
    private ISession mSession = null;

    /** les paramtres temporaires */
    private TaskData mDatas;

    /** le parser */
    private LanguageParser mParser;

    /** Le template pour le rapport de classe */
    private String mClassTemplate;

    /** Les chemins vers les JSPs si il y en a */
    private List mJspPaths;

    /**
     * Constructeur.
     * @param pParser le parser.
     * @param pConfiguration configuration du framework.
     * @param pAudit audit encadrant l'excution.
     * @param pSession la session de persistance utilise par la tche.
     * @param pDatas la liste des paramtres temporaires du projet
     * @param pTaskName le nom de la tache (pour diffrencier java et cpp)
     * @param pClassTemplate le template  utiliser
     * @throws JrafDaoException si une session de peristance ne peut tre cre.
     */
    public McCabePersistor(LanguageParser pParser, final McCabeConfiguration pConfiguration, final AuditBO pAudit, final ISession pSession, TaskData pDatas, String pTaskName, String pClassTemplate)
        throws JrafDaoException {
        mSession = pSession;
        mConfiguration = pConfiguration;
        mAudit = pAudit;
        mDatas = pDatas;
        mProject = pConfiguration.getProject();
        mParser = pParser;
        ComponentRepository repository = new ComponentRepository(mProject, mSession);
        mAdaptator = new McCabeAdaptator(pParser, repository);
        mTaskName = pTaskName;
        mClassTemplate = pClassTemplate;
        ListParameterBO jsps = (ListParameterBO) mProject.getParameter(ParametersConstants.JSP);
        if (null != jsps) { // Il s'agit d'un projet J2EE
            mJspPaths = BuildProjectPath.buildProjectPath((String) pDatas.getData(TaskData.VIEW_PATH), jsps.getParameters());
        }
    }

    /**
     * Parse le rapport des mtriques de classe.
     * @param pFilename chemin du fichier rapport.
     * @throws Exception si un problme de parsing apparait.
     * @return le nombre de rsultats de niveau classe
     * @roseuid 42B976100269
     */
    public int parseClassReport(final String pFilename) throws Exception {
        mReportFileName = pFilename;
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_class") + mReportFileName);
        // Rcuprer les beans issus du rapport de classes McCabe
        CSVParser csvparser = new CSVParser(McCabeMessages.getString("csv.config.file"));
        Collection classResults = csvparser.parse(McCabeMessages.getString(mClassTemplate), pFilename);
        // Ajout de la volumtrie dans les beans
        Iterator it = classResults.iterator();
        McCabeQAClassMetricsBO bo = null;
        while (it.hasNext()) {
            // On adapte chaque bean issu du rapport
            bo = (McCabeQAClassMetricsBO) it.next();
            bo.setAudit(mAudit);
            bo.setTaskName(mTaskName);
            // On nettoie le nom du composant si celui-ci a t mal remont du rapport,
            // en effet, quand le sparateur de valeur est la virgule et qu'une valeur contient
            // une virgule, alors cette valeur est place entre guillemets
            bo.setComponentName(ToolsReportsUtility.clearReportName(bo.getComponentName()));
            // Le bean est adapt, c'est--dire qu'on le lie  son composant
            // Si il s'agit d'une page JSP, on a stocker une partie de
            // ses rsulats dans la map mJspResults
            McCabeQAJspMetricsBO jspMetrics = (McCabeQAJspMetricsBO) mJspResults.get(bo.getComponentName());
            boolean isJSP = (null != jspMetrics);
            if (isJSP) {
                // On ne fait pas persister le bo en tant que Classe
                it.remove();
                jspMetrics.setClassMetrics(bo);
                if (null != jspMetrics.getComponent()) {
                    mJspResults.put(bo.getComponentName(), jspMetrics);
                }
            } else {
                // Sinon on l'adapte  une classe
                mNumberOfClasses++;
                mAdaptator.adaptClassResult(bo);
            }
            if (null == bo.getComponent() && !isJSP) {
                // La classe n'a pu persister, on ne fait pas persister 
                // le bo et on lance un warning
                it.remove();
                LOGGER.warn(McCabeMessages.getString("logs.warn.incorrect_class_name", bo.getComponentName()));
            }
        }
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_database"));
        // On sauvegarde le mesures sur les classes
        MeasureDAOImpl.getInstance().saveAll(mSession, classResults);
        // On sauvegarde le mesures sur les JSP si il y en a
        MeasureDAOImpl.getInstance().saveAll(mSession, mJspResults.values());
        mSession.commitTransactionWithoutClose();
        mSession.beginTransaction();
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_end"));
        return classResults.size();
    }

    /**
     * Parse le rapport des mtriques de mthodes.
     * @param pFilename chemin du fichier rapport.
     * @param pDatas les donnes temporaires
     * @throws Exception si un problme de parsing apparat.
     * @return le nombre de rsultats de niveau mthode rcuprs.
     * @roseuid 42B9761F015F
     */
    public int parseMethodReport(final String pFilename, TaskData pDatas) throws Exception {
        mReportFileName = pFilename;
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_method") + mReportFileName);
        // Rcuprer les beans issus du rapport de mthodes McCabe
        CSVParser csvparser = new CSVParser(McCabeMessages.getString("csv.config.file"));
        Collection methodResults = csvparser.parse(McCabeMessages.getString("csv.template.method"), pFilename);

        McCabeQAMethodMetricsBO bo = null;
        String name = null;
        int count = 0;
        // On cre une nouvelle collection contenant les beans  faire effectivement persister
        List resultsToPersist = new ArrayList();
        // Adaptation des beans
        Iterator it = methodResults.iterator();
        while (it.hasNext()) {
            bo = (McCabeQAMethodMetricsBO) it.next();
            it.remove();
            name = bo.getComponentName();
            // On nettoie le nom du composant si celui-ci a t mal remont du rapport,
            // en effet, quand le sparateur de valeur est la virgule et qu'une valeur contient
            // une virgule, alors cette valeur est place entre guillemets
            name = ToolsReportsUtility.clearReportName(name);
            bo.setComponentName(name);
            String className = mParser.getParentName(name);
            // On vrifie que la mthode n'est pas contenue dans les classes  exclure
            // Et que la classe n'est pas nulle (sinon c'est signe d'une dfaillance McCabe)
            bo.setAudit(mAudit);
            String absoluteFileName = bo.getFilename();
            // Le nom du fichier est mis en relatif par rapport  la racine du projet
            String viewPath = (String) pDatas.getData(TaskData.VIEW_PATH);
            bo.setFilename(FileUtility.getRelativeFileName(absoluteFileName, viewPath));
            bo.setTaskName(mTaskName);
            // Le bean est adapt, c'est--dire qu'on le lie  son composant
            // A un composant JSP si le nom du fichier se termine par .jsp
            if (bo.getFilename().endsWith(JSP_EXTENSION) && null != mJspPaths) {
                String componentName = bo.getComponentName().replaceFirst(JSP_METHOD_REGEXP, "");
                // On nettoie le nom pour enlever le "_jsp"
                bo.setComponentName(ToolsReportsUtility.clearJspName(componentName));
                McCabeQAJspMetricsBO jspMetrics = new McCabeQAJspMetricsBO(bo);
                // On rcupre le rpertoire racine contenant les JSP
                File rootDir = null;
                String relativeRootDir = null;
                int id = 0;
                for (int i = 0; i < mJspPaths.size() && null == relativeRootDir; i++) {
                    rootDir = new File((String) mJspPaths.get(i));
                    if (null != FileUtility.findFileWithPathSuffix(rootDir, absoluteFileName)) {
                        relativeRootDir = FileUtility.getRelativeFileName(rootDir.getAbsolutePath(), viewPath);
                        id = i;
                    }
                }
                if (null != relativeRootDir) {
                    mAdaptator.adaptJspResult(jspMetrics, relativeRootDir, id);
                }
                if (null == jspMetrics.getComponent()) {
                    // On lance un warning car le composant n'a pu tre pars
                    LOGGER.warn(McCabeMessages.getString("logs.warn.incorrect_jsp_method_name", jspMetrics.getComponentName()));
                } else {
                    // On enregistre le nom du composant avant la suppression du "_jsp"
                    // pour pouvoir comparer correctement.
                    mJspResults.put(componentName, jspMetrics);
                }
            } else {
                // A une mthode java
                mAdaptator.adaptMethodResult(bo);
                if (null == bo.getComponent()) {
                    // On lance un warning car le composant n'a pu tre pars
                    LOGGER.warn(McCabeMessages.getString("logs.warn.incorrect_method_name", bo.getComponentName()));
                } else {
                    mNumberOfMethods++;
                    count++;
                    resultsToPersist.add(bo);
                }
            }
        } // La collection des rsultats de mthodes est persiste
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_database"));
        MeasureDAOImpl.getInstance().saveAll(mSession, resultsToPersist);
        mSession.commitTransactionWithoutClose();
        mSession.beginTransaction();
        LOGGER.info(McCabeMessages.getString("logs.debug.report_parsing_end"));
        return methodResults.size();
    }

    /**
     * Cre et fait persister les rsultats de niveau projet.
     * @roseuid 42E09E5201AB
     */
    public void persistProjectResult() {
        LOGGER.info(McCabeMessages.getString("logs.debug.project_database"));
        McCabeQAProjectMetricsBO metrics = new McCabeQAProjectMetricsBO();
        // Cration des mtriques de niveau projet
        metrics.setComponent(mProject);
        metrics.setAudit(mAudit);
        metrics.setTaskName(mTaskName);
        metrics.setNumberOfClasses(new Integer(mNumberOfClasses));
        metrics.setNumberOfMethods(new Integer(mNumberOfMethods));
        try {
            MeasureDAOImpl.getInstance().create(mSession, metrics);
        } catch (JrafDaoException e) {
            LOGGER.error(e, e);
        }
    }
}
