

package com.airfrance.squalix.tools.umlquality;

import java.io.File;
import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.airfrance.squalix.util.file.FileUtility;

/**
 * Espace de travail utilis pour la gnration des rapport UMLQuality
 * UMLQuality utilise un rpertoire pendant son excution, ce rpertoire
 * sert   gnrer les rapports.
 * Le rpertoire est transitoire.
 */
public class UMLQualityWorkSpace {
   
    /**
     * Logger
     */
    private static final Log LOGGER = LogFactory.getLog(UMLQualityWorkSpace.class);
   
    /** Rpertoire des rapports */
    private File mReportDirectory;
    
    
    /**
     * Constructeur
     * @param pRootDir rpertoire racine
     * @throws IOException si erreur de cration des rpertoires
     */
    public UMLQualityWorkSpace(File pRootDir) throws IOException {
        // Cration du rpertoire correspondant
        mReportDirectory = pRootDir;
        createDirectory(mReportDirectory);
          
    }
   
    /**
     * Cration du rpertoire
     * @param pDirectory rpertoire  crer
     * @throws IOException si la cration ne peut se faire
     */
    private void createDirectory(File pDirectory) throws IOException {
        if (false==pDirectory.mkdirs()) {
            String message = UMLQualityMessages.getString("error.create_directory", pDirectory.getAbsolutePath());
            LOGGER.error(message);
            throw new IOException(message);
        }
    }
    
    /**
     * Nettoyage des rprtoires
     * Le rpertoire racine est dtruit rcursivement
     */
    public void cleanup() {
        // Effacement du rpertoire de rapport
        FileUtility.deleteRecursively(mReportDirectory);
    }

    /**
     * Recherche des rapports
     * @return rapports au format XML gnrs par CppTest
     */
    public Collection getReportFiles() {    
        // Recherche des rapports
        
        String []suffixsReportName=(UMLQualityMessages.getString("reports.end.file.name")).split(",");
        
        UMLQualityFileFilter filter =new UMLQualityFileFilter(suffixsReportName);
        HashSet fileList = new HashSet();
        Collection files = FileUtility.createRecursiveListOfFiles(getReportDirectory(),filter,fileList);
        return files;
    }
    
    /**
     * @return rpertoire de gnration de rapport
     */
    public File getReportDirectory() {
        return mReportDirectory;
    }


}
