package com.airfrance.squalix.util.parser;

import com.airfrance.squalecommon.enterpriselayer.businessobject.component.JspBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.PackageBO;
import com.airfrance.squalecommon.enterpriselayer.businessobject.component.ProjectBO;
import com.airfrance.squalix.tools.compiling.jsp.configuration.JspCompilingConfiguration;

/**
 * Parse les noms de type j2ee et les remplace par les objets correspondants.<br/> 
 */
public class J2EEParser extends JavaParser {

    /** Nom du package racine contenant les JSPs*/
    public static final String ROOT_PACKAGE = "{jsp}";

    /**
     * Constructeur
     * 
     * @param pProject le projet  auditer
     */
    public J2EEParser(ProjectBO pProject) {
        super(pProject);
    }
    
    /**
     * Cre le composant JSP dsign par les paramtres.
     * 
     * @param pJspName le nom de la classe JSP
     * @param pFileName le nom absolu du fichier  partir du projet
     * @param pDirectoryPath le nom absolu du rpertoire racine contenant les pages JSP
     * @param pId l'index du rpertoire source des JSPs dans les paramtres du projet
     * @return le JspBO dsign par les paramtres.
     */
    public JspBO getJsp(String pJspName, String pFileName, String pDirectoryPath, int pId) {
        JspBO jsp = new JspBO(pJspName);
        jsp.setFileName(pFileName);
        String packageName = convertIntoPackageName(pFileName, pDirectoryPath);
        // On va dcomposer le nom du fichier pour crer une hierarchie de package
        // dont la racine est un package par dfaut pour les JSP + le rpertoire indiquant
        // l'index du rpertoire source
        String absolutePackageName = ROOT_PACKAGE + "." + JspCompilingConfiguration.FIRST_PACKAGE;
        if(pId > 0) {
            absolutePackageName += pId;
        }
        if(packageName.length() > 0) {
            absolutePackageName += "." + packageName;
        }
        PackageBO packageBO = getPackage(absolutePackageName);
        // on ne vrifie pas que packageBO est nul puisqu'il aura toujours le package par dfaut
        jsp.setParent(packageBO);
        return jsp;
    }

    /**
     * @param pFileName le nom du fichier  transformer
     * @param pDirectoryPath le nom absolu du rpertoire racine  partir duquel le
     * nom du package doit tre construit
     * @return le nom du fichier transform en nom de package
     */
    private String convertIntoPackageName(String pFileName, String pDirectoryPath) {
        String packageName = "";
        // On coupe le nom du fichier
        packageName = pFileName.replaceFirst("[/\\\\]+[^/\\\\]+.jsp$", "");
        // On coupe le chemin du fichier jusqu'au rpertoire racine
        packageName = packageName.replaceFirst(pDirectoryPath + "[/\\\\]*", "");
        // On remplace les "/" par des "."
        packageName = packageName.replaceAll("/", ".");
        return packageName;
    }

}
