package com.airfrance.squaleide.preferences;

import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.IWorkbench;
import com.airfrance.squaleide.SqualePlugin;
import com.airfrance.squaleide.util.messages.PluginMessages;

import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Cree la page des prfrences
 * Cette page est utile pour le cas des projets qui n'ont pas un hierarchie dans SQUALE identique
 *  celle du workspace (cas des projets en local).
 */
public class SqualePreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    /** Chemins des rpertoires sources (jsp et java) dans l'archive */
    public static final String P_LIST = "listPreference";
    
    /** Sparateur des sources */
    public static final String SRC_SEPARATOR = ";";
    
    /** Liste pour ajouter/supprimer des chemins */
    protected AddRemoveListFieldEditor srcList;

    /**
     * Constructeur par dfaut 
     */
    public SqualePreferencePage() {
        super(GRID);
        setPreferenceStore(SqualePlugin.getDefault().getPreferenceStore());
        setDescription(PluginMessages.getString("preferences.title"));
        initializeDefaults();
    }
    /**
     * Change les valeurs par dfaut.
     */
    private void initializeDefaults() {
        IPreferenceStore store = getPreferenceStore();
        store.setDefault(P_LIST, "");
    }

    /** 
     * Cree les champs d'dition de la page de prfrence
     * @see org.eclipse.jface.preference.FieldEditorPreferencePage#createFieldEditors()
     */
    public void createFieldEditors() {
        srcList = new AddRemoveListFieldEditor(P_LIST, "", getFieldEditorParent());
        srcList.setSeparator(SRC_SEPARATOR);
        srcList.setLabelText(PluginMessages.getString("preferences.textfield_label"));
        srcList.setAddButtonText(PluginMessages.getString("preferences.add_button_label"));
        srcList.setRemoveButtonText(PluginMessages.getString("preferences.remove_button_label"));
        addField(srcList);
    }

    /** 
     * Initialise les prfrences avec les valeurs par dfaut
     * @param workbench {@inheritDoc}
     * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
     */
    public void init(IWorkbench workbench) {
        setPreferenceStore(SqualePlugin.getDefault().getPreferenceStore());
    }
    /**
     * Sauvegarde les prfrences
     * @return {@inheritDoc}
     * @see org.eclipse.jface.preference.IPreferencePage#performOk()
     */
    public boolean performOk() {
        srcList.store();
        return true;
    }
}