package com.airfrance.squaleide.xml;

import java.util.ArrayList;
import java.util.List;

import com.airfrance.squaleide.util.PluginUtils;

/**
 * Objet reprsentant le fichier XML de l'export IDE
 */
public class IdeActionPlan {
    
    /** Le nom de l'application */
    private String mApplicationName;
    
    /** La date de l'audit */
    private String mAuditDate;
    
    /** Le label de l'audit */
    private String mLabel;
    
    /** Le workspace du projet */
    private String mWorkspace;
    
    /** La liste des transgressions  corriger */
    private List mTransgressions;
    
    

    /**
     * Constructeur par dfaut
     */
    public IdeActionPlan() {
        mApplicationName = "";
        mAuditDate = "";
        mLabel = "";
        mWorkspace = "";
        mTransgressions = new ArrayList(0);
    }

    /**
     * @return le nom de l'application
     */
    public String getApplicationName() {
        return mApplicationName;
    }

    /**
     * @return la date de l'audit
     */
    public String getAuditDate() {
        return mAuditDate;
    }

    /**
     * @return le label de l'audit
     */
    public String getLabel() {
        return mLabel;
    }

    /**
     * @return la liste des trangressions
     */
    public List getTransgressions() {
        return mTransgressions;
    }

    /**
     * @return le workspace du projet
     */
    public String getWorkspace() {
        return mWorkspace;
    }

    /**
     * @param pApplicationName le nom de l'application
     */
    public void setApplicationName(String pApplicationName) {
        mApplicationName = pApplicationName;
    }

    /**
     * @param pAuditDate la date de l'audit
     */
    public void setAuditDate(String pAuditDate) {
        mAuditDate = pAuditDate;
    }

    /**
     * @param pLabel le label de l'audit
     */
    public void setLabel(String pLabel) {
        mLabel = pLabel;
    }

    /**
     * @param pTransgressions la liste des transgressions
     */
    public void setTransgressions(List pTransgressions) {
        mTransgressions = pTransgressions;
    }

    /**
     * @param pWorkspace le workspace
     */
    public void setWorkspace(String pWorkspace) {
        mWorkspace = pWorkspace;
    }
    
    /**
     * Ajoute une transgression  la liste des transgressions
     * @param pTransgression la transgression  rajouter
     */
    public void addTransgression(IdeTransgression pTransgression) {
        // On transforme les chemins des fichiers pour qu'ils soient relatifs
        // au workspace :
        // Il se peut que le "/" manque dans la description du rpertoire, alors on le rajoute
        // pour pouvoir remplacer correctement le workspace
        if(pTransgression.getInFolder().length() > 0 && !pTransgression.getInFolder().startsWith("/")) {
            pTransgression.setInFolder("/" + pTransgression.getInFolder());
        }
        pTransgression.setInFolder(PluginUtils.replaceFirst(getWorkspace(), pTransgression.getInFolder()));
        // On supprime le premier "/" du chemin du rpertoire
        if(pTransgression.getInFolder().startsWith("/")) {
            pTransgression.setInFolder(pTransgression.getInFolder().substring(1));
        }
        mTransgressions.add(pTransgression);
    }

}
